
(defvar ripple-mode-hook nil)
(defvar ripple-mode-map
  (let ((ripple-mode-map (make-keymap)))
    (define-key ripple-mode-map "\C-j" 'newline-and-indent)
    ripple-mode-map)
  "Keymap stuff")

(add-to-list 'auto-mode-alist '("\\.rip\\'" . ripple-mode))

(defconst ripple-font-lock-keywords-1
  (list
   '("\\<\\(->\\|defun\\|els\\(?:e\\|if\\)\\|for\\|if\\|while\\)\\>" . font-lock-builtin-face)
   '("\\('.*'\\)" . font-lock-constant-face))
  "A small set of highlighting keywords and symbols")

(defconst ripple-font-lock-keywords-2
  (append ripple-font-lock-keywords-1
	  (list
	   '("\\<\\(break\\|:\\|print\\(?:f\\|ln\\)?\\|re\\(?:ad\\|turn\\)\\)\\>" . font-lock-keyword-face)
	   '("\\<\\(true\\|false\\)\\>" . font-lock-constant-face)
	   )))

(defconst ripple-font-lock-keywords-3
  (append ripple-font-lock-keywords-2
	  (list
	   '("\\<\\(bool\\|float\\|int\\|string\\|void\\)\\>" . font-lock-type-face)
	   '("\\(\-\>\\)" . font-lock-builtin-face))))

(defvar ripple-font-lock-keywords ripple-font-lock-keywords-3 "ripple")

(defun ripple-indent-line ()
  "Indenting function for language"
  (interactive)
  (beginning-of-line)
  (if (bobp)
      (indent-line-to 0)
    (let ((not-indented t) cur-indent)
      (if (looking-at "\\(.*\\(elsif\\|else\\)\\|^[ \t]*;\\)")
	  (progn
	    (save-excursion
	      (forward-line -1)
	      (setq cur-indent (- (current-indentation) 3)))
	    (if (< cur-indent 0)
		(setq cur-indent 0)))
	(save-excursion
	  (while not-indented
	    (forward-line -1)
	    (if (looking-at "^[ \t]*;")
		(progn
		  (setq cur-indent (current-indentation))
		  (setq not-indented nil))
	      (if (looking-at ".*\-\>")
		  (progn
		    (setq cur-indent (+ (current-indentation) 3))
		    (setq not-indented nil))
		(if (bobp)
		    (setq not-indented nil)))))))
      (if cur-indent
	  (indent-line-to cur-indent)
	(indent-line-to 0)))))

;; fix comments... 
(defvar ripple-mode-syntax-table
  (let ((st (make-syntax-table)))
    (modify-syntax-entry ?_ "w" st)
    (modify-syntax-entry ?\# "< 12" st)
    (modify-syntax-entry ?\n "> " st)
    st)
  "Default syntax table for ripple")

(defun ripple-mode ()
  (interactive)
  (kill-all-local-variables)
  (use-local-map ripple-mode-map)
  (set-syntax-table ripple-mode-syntax-table)
  (set (make-local-variable 'font-lock-defaults) '(ripple-font-lock-keywords))
  (set (make-local-variable 'indent-line-function) 'ripple-indent-line)
  (setq major-mode 'ripple-mode)
  (setq mode-name "ripple")
  (run-hooks 'ripple-mode-hook))

(provide 'ripple-mode)