# -*- coding: utf-8 -*-

##
# This method takes an expression
# and decides what to do with it
# based on the expressions class.
##
def get_expression (expression, context=nil)
  if expression.class == Sstring
    "\"#{expression.value}\""
  elsif expression.class == Numeric
    expression.value
  elsif expression.class == Identifier
    context.get_variable("#{expression.name}")
  elsif expression.class == Booltype
    expression.name
  elsif expression.class == AritmExpr
    infix = convert_aritm_to_infix(expression, context)
    infix
  elsif expression.class == PredExpr || 
      expression.class == RelPredExpr || 
      expression.class == NpredExpr
    infix = convert_logic_to_infix(expression, context)
    infix
  elsif expression.class == FuncCall
    handle_FuncCall(expression, context)
  end
end


###
# Handler for if statements. This function 
# is used to iterate in a logical way through 
# all steps involved in an if statement.
###
def handle_IfStmt(statement, context)

  puts statement if $debug

  value = nil

  if (evaluate(statement.condition, context) == true) then
    puts "Condition true" if $debug
    statement.statements.each do |stm|
      value = evaluate(stm, context)
    end
  else
    done = false
    if (statement.elsif_list != nil) then
      statement.elsif_list.each do |els|
        break if (done == true)
        if (evaluate(els.condition, context) == true) then
          done = true
          els.statements.each do |estm|
            value = evaluate(estm, context)
          end
        end
      end
    end
    if (done == false && statement.else != nil) then
      statement.else.statements.each do |elstm|
        value = evaluate(elstm, context)
      end
    end
  end
  value
end

###
# This method takes care of while statements.
# This is done by iterating through each 
# statement in the while statement and for
# each iteration check if the condition is
# true or false by evaluating the condition
# in the context of the source of the 
# statement.
###
def handle_WhileStmt(stmt, context)

  puts stmt if $debug
  introduced_vars = []

  broken = false
  while (evaluate(stmt.condition, context) == true && broken == false)
    stmt.statements.each do |wstmt|
      if wstmt.class == BreakStmt then
        broken = true
        break
      end
      value = evaluate(wstmt, context)
      if value == "Break" then
        broken = true
        break
      end
      if wstmt.class == AssignStmt then
        puts "assign stmt" if $debug
        introduced_vars << wstmt.identifier.name
      end
    end
    print "Introduced vars:\n#{introduced_vars}\n" if $debug
    introduced_vars.each do |var|
      context.unset_variable("#{var}")
    end
  end

end

##
# Handler for for statements. Iterates through
# each statement of the for statement and
# for each iteration evaluates the condition.
##
def handle_ForStmt(stmt, context)

  identifier = stmt.identifier.name
  condition = stmt.condition
  increment = nil
  broken = false

  #Get the value with wich to increment with
  if(stmt.inc_value.class == Identifier) then
    increment = stmt.inc_value.name
    increment = context.get_variable("#{increment}")
  elsif (stmt.inc_value.class == Numeric) then
    increment = stmt.inc_value.value
  end

  # Iterate through each statement in the for loop 
  # while the condition is true, and then increment.
  while(evaluate(condition, context) == true && broken == false)
    stmt.statements.each do |fstmt|
      if(fstmt.class == BreakStmt) then
        broken = true
        break
      end
      value = evaluate(fstmt, context)
      if value == "Break" then
        broken = true
        break
      end
    end

    old_value = context.get_variable("#{identifier}")
    test_value = old_value + increment
    context.set_variable("#{identifier}", test_value)

    if(evaluate(condition, context) == false) then
      context.set_variable("#{identifier}", old_value)
      break
    end
  end

end

##
# This method handles read statements.
# Depending of parameters content is read
# either from stdin or a file.
##
def handle_ReadStmt(stmt, context)

  source = stmt.source
  target = stmt.target.name
  read_text = ""

  if(source == "stdin") then
    # This is suspicious. I had to explicitly tell 
    # ruby that i wanted $stdin, should be default.
    read_text = $stdin.gets
  elsif (source.class == Sstring)
    f = File.new(source.value)
    read_text = f.read()
    f.close()
  end

  read_text = read_text.gsub("\"", '\"')
  context.set_variable("#{target}", read_text.chomp)

end

##
# Method for handling printing to 
# file. If the file exists it 
# defaults to appending to the 
# file, else it creates a new.
##
def handle_PrintfStmt(stmt, context)

  source = stmt.source.name
  target = stmt.target.value
  f = nil

  if (File.exists?(target)) then
    puts "file exists"
    f = File.open(target, 'a')
  else
    puts "creating new file"
    f = File.open(target, 'w')
  end

  f.puts(context.eval("get_variable(\"#{source}\")").gsub("\"", ""))
  f.close()

end

##
# Method for function calls. Since
# all functions are stored within the
# context of the calling object, it
# simply asks for a new instance of the
# function and returns the value of the 
# executed function.
##
def handle_FuncCall(stmt, context)

  if context.has_function(stmt.name.name) then
    arg_list = []
    stmt.arguments.each do |arg|
      arg_list << evaluate(arg, context)
    end
    context.get_function(stmt.name.name).call(arg_list)
  else
    raise Exception, "There's no function with name: \"#{stmt.name.name}\""
  end

end

