# -*- coding: utf-8 -*-

##
# This file contains functions to convert 
# arithmetic and logic expressions in prefix 
# notation to the corresponding ones in infix.
##

##
# This method receives an expression
# in prefix notation and returns 
# a calculated value.
##
def convert_aritm_to_infix(expr, context)
  head = MathNode.new(expr.operator, expr.operand, 
                      expr.operand1, context)
  result = head.collect
  result
end

##
# This class will, when instantiated, create
# a tree of nodes which contains all operantions
# that need to be done to calculate a value.
##
class MathNode

  attr_accessor :operator, :operand1, :operand2, :left, 
  :right, :context

  def initialize(operator, operand1, operand2, context)
    @operator, @operand1, @operand2, @context = [operator, operand1, 
                                                 operand2, context]
    left, right = [nil,nil]

    if(operand1.class == AritmExpr) then
      @left = MathNode.new(operand1.operator, operand1.operand, 
                           operand1.operand1, context)
    else
      @left = operand1
    end

    if(operand2.class == AritmExpr) then
      @right = MathNode.new(operand2.operator, operand2.operand, 
                            operand2.operand1, context)
    else 
      @right = operand2
    end
  end

  # Since all mathematical operations contain
  # two operands we first collect the left node
  # and then put an operator and finally 
  # collect the right node. 
  def collect
    retStr = ""
    if(left != nil && left.class == MathNode) then
      retStr += "#{left.collect}"
    elsif(left.class == Numeric)
      retStr += "#{left.value}"
    elsif(left.class == Identifier)
      retStr += context.get_variable("#{left.name}").to_s
    else
      retStr += "#{evaluate(left, context)}"
    end

    if(operator.to_s == "^")
      retStr += "**"
    else
      retStr += operator.to_s
    end

    if(right != nil && right.class == MathNode) then
      retStr += "#{right.collect}"
    elsif(right.class == Numeric)
      retStr += "#{right.value}"
    elsif(right.class == Identifier)
      retStr += context.get_variable("#{right.name}").to_s
    else
      retStr += "#{evaluate(right, context)}"
    end

    puts retStr if $debug
    puts "#{eval(retStr)}" if $debug

    #Evaluating resulting string to make sure we return a valid number
    eval("#{retStr}")
  end

end

##
# This method receives a logic expression
# in prefix and returns a calculated value.
##
def convert_logic_to_infix(expr, context)
  if(expr.class != NpredExpr) then
    head = LogNode.new(expr.operator, expr.operand, 
                       expr.operand1, context)
  else
    head = LogNode.new(expr.operator, expr.operand, 
                       nil, context)
  end
  result = head.collect
  result
end

##
# This class represents a node in a tree
# of logic expressions. 
##
class LogNode

  attr_accessor :operator, :operand1, :operand2, :left, 
  :right, :context

  def initialize(operator, operand1, operand2=nil, context)
    @operator, @operand1, @operand2, @context = [operator, operand1, 
                                                 operand2, context]
    @left,@right = [nil, nil]

    if(operand1.class == RelPredExpr || operand1.class == PredExpr)
      @left = LogNode.new(operand1.operator, operand1.operand, 
                          operand1.operand1, context)
    elsif(operand1.class == NpredExpr)
      @left = LogNode.new(operand1.operator, operand1.operand, 
                          nil, context)
    else
      @left = operand1
    end

    if(operand2.class == RelPredExpr || operand2.class == PredExpr)
      @right = LogNode.new(operand2.operator, operand2.operand, 
                           operand2.operand1, context)
    elsif(operand2.class == NpredExpr)
      @right = LogNode.new(operand2.operator, operand2.operand, 
                           nil, context)
    else
      @right = operand2
    end
  end

  # This method needs to handle two basic
  # cases. Either this node is a logic 
  # expression with the operator "!", 
  # which means that there only is one
  # operand, or else there are two
  # operands.
  def collect
    retStr = ""
    
    # The case with only one operand
    if(self.operator == "!") then
      retStr += self.operator.to_s
      if (left.class == LogNode)
        retStr += "#{left.collect}"
      elsif (left.class == AritmExpr) 
        retStr += convert_aritm_to_infix(left, context).to_s
      elsif (left.class == Numeric)
        retStr += left.value.to_s
      elsif (left.class == Identifier)
        retStr += "get_variable(\"#{self.operand1.name.to_s}\")"
      elsif (left.class == Booltype)
        retStr += "#{self.operand1.name}"
      else
        retStr += "#{self.operand1}"
      end

    # The case with two operands.
    else
      if(left != nil && left.class == LogNode) then
        retStr += "#{left.collect}"
      elsif (left.class == AritmExpr)
        retStr += convert_aritm_to_infix(left, context).to_s
      elsif(left.class == Numeric && self.operator != "!")
        retStr += left.value.to_s
      elsif(left.class == Identifier && self.operator != "!")
        retStr += "get_variable(\"#{left.name.to_s}\")"
      elsif(left.class == Booltype && self.operator != "!")
        p "sadf"
        retStr += left.name.to_s
      elsif(self.operator != "!")
        retStr += "#{self.operand1}"
      end
      
      if(self.operator == "&")
        retStr += "&&"
      elsif(self.operator == "|")
        retStr += "||"
      elsif(self.operator == "=")
        retStr += "=="
      else
        retStr += operator
      end
      
      if(right != nil && right.class == LogNode) then
        retStr += "#{right.collect}"
      elsif(self.class != NpredExpr) then
        if(right.class == Numeric)
          retStr += right.value.to_s
        elsif(right.class == AritmExpr)
          retStr += convert_aritm_to_infix(right, context).to_s
        elsif(right.class == Identifier)
          retStr += "get_variable(\"#{right.name.to_s}\")"
        elsif(right.class == Booltype)
          retStr += "#{right.name}"
        end
      end
      
    end
   
    puts "should eval: #{retStr}" if $debug
    context.eval("(#{retStr})")
  end

end
